<?php
/**
 * License Keys main controller
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys
 * @copyright Copyright (c) 2020, We Hear You 2, Inc.
 */

require_once PLUGINDIR . 'license_keys' . DS . 'vendors' . DS . 'phpspreadsheet'. DS .'autoload.php';
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use Blesta\Core\Util\Common\Traits\Container;

class AdminMain extends LicenseKeysController {

    use Container;

    public function preAction()	{
		parent::preAction();
		$this->structure->setDefaultView(APPDIR);
		$this->structure->setView(null, $this->orig_structure_view);

		$this->requireLogin();

		$this->uses(['LicenseKeys.LicenseKeysList','Clients']);
		Language::loadLang('admin_main', null, PLUGINDIR . 'license_keys' . DS . 'language' . DS);

	}

	/**
	 * Renders the Extensions Info widget
	 */
	public function index() {
		$page = (isset($this->get[0]) ? (int)$this->get[0] : 1);
		$sort = (isset($this->get['sort']) ? $this->get['sort'] : 'client_id_code');
		$order = (isset($this->get['order']) ? $this->get['order'] : 'desc');
        $clients = $this->Clients->getAll("active",null);
        $this->set("clients",$clients);

        $filter['client_id_code'] = (isset($this->get['client_id_code'])) ? $this->get['client_id_code'] : "";
        $filter['client_id'] = (isset($this->get['client_id'])) ? $this->get['client_id'] : "";

        $filter['extension_name'] = (isset($this->get['product_name']) && $this->get['product_name']!=='all') ? $this->get['product_name'] : "";
		$filter['subscribed'] = (isset($this->get['subscribed']) && $this->get['subscribed']!=='all') ? $this->get['subscribed'] : "";
        $filter['license_key'] = (isset($this->get['license_key']) ? $this->get['license_key'] : '');
        $filter['blesta_license_key'] = (isset($this->get['blesta_license_key']) ? $this->get['blesta_license_key'] : '');

        $filter['from'] = (isset($this->get['from']) ? $this->get['from'] : '');
		$filter['to'] = (isset($this->get['to']) ? $this->get['to'] : '');
		$infos = $this->LicenseKeysList->getList($page, [$sort => $order],$filter);
        $total_results = $this->LicenseKeysList->getListCount($filter);

        $product_name = '';
        $subscribed = '';
		if (isset($this->get['product_name'])) {
		    $product_name=$this->get['product_name'];
        }
        if (isset($this->get['subscribed'])) {
            $subscribed=$this->get['subscribed'];
        }
		$this->set('selected_product_name',$product_name);
		$this->set('selected_subscribed',$subscribed);
		$this->set('from',$filter['from']);
		$this->set('to',$filter['to']);
        $this->set('client_id',$filter['client_id']);
        $this->set('client_id_code',$filter['client_id_code']);
        $this->set('license_key',$filter['license_key']);
        $this->set('blesta_license_key',$filter['blesta_license_key']);
        $this->set('infos', $infos);
		$this->set('sort', $sort);
		$this->set('order', $order);
		$this->set('negate_order', ($order == 'asc' ? 'desc' : 'asc'));

		// Set pagination parameters, set group if available
		$params = ['sort' => $sort,'order' => $order];
        if ($filter['client_id_code']) {
            $params['client_id_code'] = $filter['client_id_code'];
        }

        if ($filter['client_id']) {
            $params['client_id'] = $filter['client_id'];
        }

		if ($filter['extension_name']) {
			$params['product_name'] = $filter['extension_name'];
		}

		if ($filter['subscribed']) {
			$params['subscribed'] = $filter['subscribed'];
		}

		if ($filter['from']) {
			$params['from']=$filter['from'];
			$params['to']=$filter['to'];
		}

        if ($filter['license_key']) {
            $params['license_code'] = $filter['license_key'];
        }

        if ($filter['blesta_license_key']) {
            $params['blesta_license_code'] = $filter['blesta_license_key'];
        }

        $logger = $this->getFromContainer('logger');
        // Overwrite default pagination settings
		$settings = array_merge(
			Configure::get('Blesta.pagination'),
			[
				'total_results' => $total_results,
				'uri' => $this->base_uri . 'plugin/license_keys/admin_main/index/[p]/',
				'params' => $params
			]
		);
		$this->setPagination($this->get, $settings);

		// Render the request if ajax
		return $this->renderAjaxWidgetIfAsync(isset($this->get[1]) || isset($this->get['sort']));
	}

	/**
	 * Remove a info
	 */
	public function delete() {
		if (isset($this->get[0])) {
			$this->LicenseKeysList->delete($this->get[0]);
			$title = Language::_('AdminMain.!success.info_removed', true);
			$title = str_replace("{id}",$this->get[0],$title);
			$this->flashMessage('message', $title);
		}

		$this->redirect($this->base_uri . 'plugin/license_keys/admin_main/index/');

	}

	public function export() {
		$index=2;
		$filter['extension_name'] = (isset($this->get['product_name']) && $this->get['product_name']!=='all') ? $this->get['product_name'] : "";
		$filter['subscribed'] = (isset($this->get['subscribed']) && $this->get['subscribed']!=='all') ? $this->get['subscribed'] : "";
		$filter['from'] = (isset($this->get['from']) ? $this->get['from'] : '');
		$filter['to'] = (isset($this->get['to']) ? $this->get['to'] : '');
        $filter['client_id'] = (isset($this->get['client_id']) ? $this->get['client_id'] : '');
        $filter['client_id_code'] = (isset($this->get['client_id_code']) ? $this->get['client_id_code'] : '');
        $filter['license_key'] = (isset($this->get['license_key']) ? $this->get['license_key'] : '');
        $filter['blesta_license_key'] = (isset($this->get['blesta_license_key']) ? $this->get['blesta_license_key'] : '');

        $infos = json_decode(json_encode($this->LicenseKeysList->getLicenseKeys($filter,array("client_id"=>"desc"))),true);

		$spreadsheet = new Spreadsheet();
		$sheet       = $spreadsheet->getActiveSheet();
		$sheet->setCellValue( 'A1', 'Client ID' );
		$sheet->setCellValue( 'B1', 'Name' );
		$sheet->setCellValue( 'C1', 'Email' );
		$sheet->setCellValue( 'D1', 'Company' );
		$sheet->setCellValue( 'E1', 'Country' );
		$sheet->setCellValue( 'F1', 'State' );
		$sheet->setCellValue( 'G1', 'Extension Name' );
		$sheet->setCellValue( 'H1', 'Version' );
		$sheet->setCellValue( 'I1', 'Installation URL' );
		$sheet->setCellValue( 'J1', 'IP Address' );
		$sheet->setCellValue( 'K1', 'Phpmillion License Key' );
        $sheet->setCellValue( 'L1', 'Blesta License Key' );
        $sheet->setCellValue( 'M1', 'Subscribed' );
		$sheet->setCellValue( 'N1', 'Created At' );
        $sheet->setCellValue( 'O1', 'Support Service Status' );
        $sheet->setCellValue( 'P1', 'Rent/Support Renewed At' );
        $sheet->setCellValue( 'Q1', 'Rent/Support Renewal Date' );

		$styleArray = array(
			'borders' => array(
				'top'    => array(
					'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
				),
				'left'   => array(
					'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
				),
				'right'  => array(
					'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
				),
				'bottom' => array(
					'borderStyle' => \PhpOffice\PhpSpreadsheet\Style\Border::BORDER_THIN,
				),
			)
		);

		$spreadsheet->getActiveSheet()->getColumnDimension( 'A' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'A1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'B' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'B1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'C' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'C1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'D' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'D1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'E' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'E1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'F' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'F1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'G' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'G1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'H' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'H1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'I' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'I1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'J' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'J1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'K' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'K1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'L' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'L1' )->applyFromArray( $styleArray );
		$spreadsheet->getActiveSheet()->getColumnDimension( 'M' )->setAutoSize( true );
		$spreadsheet->getActiveSheet()->getStyle( 'M1' )->applyFromArray( $styleArray );
        $spreadsheet->getActiveSheet()->getColumnDimension( 'N' )->setAutoSize( true );
        $spreadsheet->getActiveSheet()->getStyle( 'N1' )->applyFromArray( $styleArray );
        $spreadsheet->getActiveSheet()->getColumnDimension( 'O' )->setAutoSize( true );
        $spreadsheet->getActiveSheet()->getStyle( 'O1' )->applyFromArray( $styleArray );
        $spreadsheet->getActiveSheet()->getColumnDimension( 'P' )->setAutoSize( true );
        $spreadsheet->getActiveSheet()->getStyle( 'P1' )->applyFromArray( $styleArray );
        $spreadsheet->getActiveSheet()->getColumnDimension( 'Q' )->setAutoSize( true );
        $spreadsheet->getActiveSheet()->getStyle( 'Q1' )->applyFromArray( $styleArray );

		foreach ($infos as $items) {
			$sheet->setCellValue( 'A' . $index, $items['client_id_code'] );
			$spreadsheet->getActiveSheet()->getStyle( 'A' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'B' . $index, $items['first_name'].' '.$items['last_name'] );
			$spreadsheet->getActiveSheet()->getStyle( 'B' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'C' . $index, $items['email'] );
			$spreadsheet->getActiveSheet()->getStyle( 'C' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'D' . $index, $items['company'] );
			$spreadsheet->getActiveSheet()->getStyle( 'D' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'E' . $index, $items['country']);
			$spreadsheet->getActiveSheet()->getStyle( 'E' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'F' . $index, $items['state'] );
			$spreadsheet->getActiveSheet()->getStyle( 'F' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'G' . $index, $items['extension_name'] );
			$spreadsheet->getActiveSheet()->getStyle( 'G' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'H' . $index, $items['version'] );
			$spreadsheet->getActiveSheet()->getStyle( 'H' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'I' . $index, $items['installation_url'] );
			$spreadsheet->getActiveSheet()->getStyle( 'I' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'J' . $index, $items['ip_address'] );
			$spreadsheet->getActiveSheet()->getStyle( 'J' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'K' . $index, $items['license_code'] );
			$spreadsheet->getActiveSheet()->getStyle( 'K' . $index )->applyFromArray( $styleArray );
            $sheet->setCellValue( 'L' . $index, $items['blesta_license_code'] );
            $spreadsheet->getActiveSheet()->getStyle( 'L' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'M' . $index, $items['subscribed'] );
			$spreadsheet->getActiveSheet()->getStyle( 'M' . $index )->applyFromArray( $styleArray );
			$sheet->setCellValue( 'N' . $index, $items['created_at'] );
			$spreadsheet->getActiveSheet()->getStyle( 'N' . $index )->applyFromArray( $styleArray );
            $sheet->setCellValue( 'O' . $index, $items['support_service_status'] );
            $spreadsheet->getActiveSheet()->getStyle( 'O' . $index )->applyFromArray( $styleArray );
            $sheet->setCellValue( 'P' . $index, $items['renewed_date'] );
            $spreadsheet->getActiveSheet()->getStyle( 'P' . $index )->applyFromArray( $styleArray );
            $sheet->setCellValue( 'Q' . $index, $items['renewal_date'] );
            $spreadsheet->getActiveSheet()->getStyle( 'Q' . $index )->applyFromArray( $styleArray );
			$index++;
		}

		$start = date_create($filter['from']);
		$start_date = $start->format("m-d-Y");
		$end = date_create($filter['to']);
		$end_date = $end->format("m-d-Y");

		$filename ="";

		if ($filter['extension_name']) {
			$filename.= "Report for Extension ".$filter['extension_name'];
		} else {
			$filename.= "Report for All extension";
		}

		if ($filter['from'] && $filter['to']) {
			$filename.=" from ".$start_date." - ".$end_date;
		} else {
			$filename.=" on all period";
		}

		//$filename = "License Keys for ". $filter['extension_name'] ? $filter['extension_name'] : "All" ." from ".$start_date." - ".$end_date.".xlsx";
		header( 'Content-type: application/vnd.ms-excel' );
		// It will be called order.xls
		header( "Content-Disposition: attachment; filename=".$filename.".xlsx");
		$writer = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter( $spreadsheet, "Xlsx" );
		ob_end_clean();
		// Write file to the browser
		$writer->save( 'php://output' );
		die();
	}

    private function getExtensionNameByProductId($id) {
        switch ($id) {
            case "1":
                $product_name = 'AddThis';
                break;
            case "2":
                $product_name = 'Advanced reports';
                break;
            case "3":
                $product_name = 'AusPost Integration';
                break;
            case "5":
                $product_name = 'Calendar';
                break;
            case "6":
                $product_name = 'CanadaPost Integration';
                break;
            case "7":
                $product_name = 'Constant Contact Integration';
                break;
            case "59" :
                $product_name = 'Delivery Scheduler';
                break;
            case "65":
                $product_name = 'Easy Slider';
                break;
            case "8":
                $product_name = 'Email Editor';
                break;
            case "4":
                $product_name = 'Enhanced AIM';
                break;
            case "9":
                $product_name = 'Enhanced Editor';
                break;
            case "11" :
                $product_name = 'Enhanced Flat Rate';
                break;
            case "10" :
                $product_name = 'FedEx Integration';
                break;
            case "12" :
                $product_name = 'HonestGreen Import Addon';
                break;
            case "13" :
                $product_name = 'HonestGreen Orders Management Addon';
                break;
            case "64" :
                $product_name = 'Interac e-Transfer';
                break;
            case "60" :
                $product_name = 'Inventory Report';
                break;
            case "14" :
                $product_name = 'MailChimp Integration';
                break;
            case "15" :
                $product_name = 'MailChimp Plus Integration';
                break;
            case "55" :
                $product_name = 'Multiple Discount Manager';
                break;
            case "16" :
                $product_name = 'Multiple Price Changer';
                break;
            case "52" :
                $product_name = 'NZ Post Integration';
                break;
            case "53" :
                $product_name = 'Online Newsletter';
                break;
            case "17" :
                $product_name = 'Option Price Lite';
                break;
            case "18" :
                $product_name = 'Option Price Pro';
                break;
            case "20" :
                $product_name = 'Option Price Link Pro';
                break;
            case "21" :
                $product_name = 'Product As Option';
                break;
            case "22" :
                $product_name = 'Purolator Integration';
                break;
            case "23" :
                $product_name = 'Pricing Calculator';
                break;
            case "50" :
                $product_name = 'Product Pricing Plus';
                break;
            case "24" :
                $product_name = 'Quick Checkout';
                break;
            case "58" :
                $product_name = 'Quick Grid Export';
                break;
            case "25" :
                $product_name = 'Quick View';
                break;
            case "47" :
                $product_name = 'Reorder';
                break;
            case "26" :
                $product_name = 'Sales By State Report';
                break;
            case "54" :
                $product_name = 'ShipEngine Integration';
                break;
            case "27" :
                $product_name = 'Special Shipping With Images';
                break;
            case "57" :
                $product_name = 'Style Changer';
                break;
            case "28" :
                $product_name = 'TaxJar Integration';
                break;
            case "29" :
                $product_name = 'Telephone Option';
                break;
            case "30" :
                $product_name = 'Total Import';
                break;
            case "31" :
                $product_name = 'UPS Integration';
                break;
            case "32" :
                $product_name = 'UPS Integration Plus';
                break;
            case "33" :
                $product_name = 'USPS Integration';
                break;
            case "34" :
                $product_name = 'VAT/Tax Id Option';
                break;
            case "48" :
                $product_name = 'Banner Statistic Manager';
                break;
            case "49" :
                $product_name = 'Brands Info';
                break;
            case "46" :
                $product_name = 'Build A Gift';
                break;
            case "36" :
                $product_name = 'Cost Plus Changer';
                break;
            case "66" :
                $product_name = 'Coupon By Locations';
                break;
            case "35" :
                $product_name = 'EasyAppointment Integration';
                break;
            case "39":
                $product_name = 'Length Calculator';
                break;
            case "41":
                $product_name = 'Weight Calculator';
                break;
            case "42":
                $product_name = 'Area Calculator';
                break;
            case "43":
                $product_name = 'Volume Calculator';
                break;
            case "45":
                $product_name = 'Liquids Calculator';
                break;
            case "61":
                $product_name = 'Customer Group Auto Assigned';
                break;
            case "63":
                $product_name = 'Customer Group Category Access';
                break;
        }
        return $product_name;
    }

}
