<?php

/**
 * License Keys Client main controller
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys
 * @copyright Copyright (c) 2020, We Hear You 2, Inc.
 */

class ClientLicenseInfo extends LicenseKeysController {

    private $client_id = null;

    public function preAction() {
        parent::preAction();
        $this->structure->setDefaultView(APPDIR);
        $this->structure->setView(null, $this->orig_structure_view);

        Language::loadLang('client_license_info', null, PLUGINDIR . 'license_keys' . DS . 'language' . DS);
        $this->client_id = $this->Session->read('blesta_client_id');
    }

    /**
     * Renders the Extensions Info widget
     */
    public function index() {

        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList','Companies']);
        $client_id = $this->client_id;

        $page = (isset($this->get[0]) ? (int)$this->get[0] : 1);
        $sort = (isset($this->get['sort']) ? $this->get['sort'] : 'extension_name');
        $order = (isset($this->get['order']) ? $this->get['order'] : 'asc');

        $license_info = $this->LicenseKeysList->getLicenseInfo($client_id,$page, [$sort => $order]);
        $total_results = $this->LicenseKeysList->getLicenseCount($client_id);

        $get_timezone = $this->Companies->getSetting(1,'timezone');
        $timezone=$get_timezone->value;
        $this->set("timezone", $timezone);
        $this->set("license_info", $license_info);
        $this->set('sort', $sort);
        $this->set('order', $order);
        $this->set('negate_order', ($order == 'asc' ? 'desc' : 'asc'));

        // Set pagination parameters, set group if available
        $params = ['sort' => $sort,'order' => $order];

        // Overwrite default pagination settings
        $settings = array_merge(
            Configure::get('Blesta.pagination_client'),
            [
                'total_results' => $total_results,
                'uri' => $this->base_uri . 'plugin/license_keys/client_license_info/index/[p]/',
                'params' => $params
            ]
        );
        $this->setPagination($this->get, $settings);

        // Render the request if ajax
        return $this->renderAjaxWidgetIfAsync(isset($this->get[1]) || isset($this->get['sort']));
    }

}
