<?php
/**
 * License Keys parent controller for all License Keys child controllers to inherit from
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys
 * @copyright Copyright (c) 2020, We Hear You 2, Inc.
 */
class LicenseKeysController extends AppController {

	/**
	 * @var The license keys mailer session key
	 */
	private $license_keys_send_emails_session_key = 'license_keys_send_emails';

    public function preAction()
    {
	    $this->structure->setDefaultView(APPDIR);

	    parent::preAction();
	    $this->requireLogin();
        // Override default view directory
	    $this->view->view = 'default';
	    $this->orig_structure_view = $this->structure->view;
	    $this->structure->view = 'default';
    }

	/**
	 * Retrieves the logged-in staff user
	 *
	 * @return stdClass An stdClass object representing the staff member
	 */
	protected function getStaff()
	{
		$this->uses(['Staff']);
		return $this->Staff->get($this->Session->read('blesta_staff_id'));
	}

	/**
	 * Makes the WYSIWYG available to the view
	 */
	protected function setWysiwyg()
	{
		// Include WYSIWYG
        $this->Javascript->setFile('blesta/ckeditor/build/ckeditor.js', 'head', VENDORWEBDIR);
	}


	/**
	 * Writes license keys send email data to session
	 *
	 * @param string $key The name of the key associated with the value
	 * @param string $value The value
	 */
	protected function write($key, $value) {
		// Fetch the current session data
		$data = [];
		$session = $this->Session->read($this->license_keys_send_emails_session_key);
		if (!empty($session)) {
			$data = (array)$session;
		}

		// Merge the given data to the session
		$this->Session->write(
			$this->license_keys_send_emails_session_key,
			array_merge($data, [$key => $value])
		);
	}

	/**
	 * Reads license keys send email data from the session
	 *
	 * @return mixed The session data stored
	 */
	protected function read() {
		return $this->Session->read($this->license_keys_send_emails_session_key);
	}

	/**
	 * Deletes the session data stored for the license keys send email
	 */
	protected function clear()	{
		$this->Session->clear($this->license_keys_send_emails_session_key);
	}
}
