<?php

use Blesta\Core\Util\Common\Traits\Container;

/**
 * Feed Reader plugin handler
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys
 * @copyright Copyright (c) 2020-2022, We Hear You 2, Inc.
 */
class LicenseKeysPlugin extends Plugin {

    use Container;

    public function __construct() {
        // Load components required by this plugin
        Loader::loadComponents($this, ['Input', 'Record','Session']);

        Language::loadLang('license_keys_plugin', null, dirname(__FILE__) . DS . 'language' . DS);
        $this->loadConfig(dirname(__FILE__) . DS . 'config.json');
    }

    /**
     * Performs any necessary bootstraping actions
     *
     * @param int $plugin_id The ID of the plugin being installed
     */
    public function install($plugin_id) {
        if (!isset($this->Record)) {
            Loader::loadComponents($this, ['Record']);
        }

        //Loader::loadModels($this, ['LicenseKeys.LicenseKeysList']);
        Configure::load('license_keys', dirname(__FILE__) . DS . 'config' . DS);
        Loader::loadModels($this, ['Emails', 'EmailGroups', 'Languages']);

        // Fetch all currently-installed languages for this company, for which email templates should be created for
        $languages = $this->Languages->getAll(Configure::get('Blesta.company_id'));

        // Add all email templates
        $emails = Configure::get('LicenseKeys.install.emails');
        foreach ($emails as $email) {
            $group = $this->EmailGroups->getByAction($email['action']);
            if ($group) {
                $group_id = $group->id;
            } else {
                $group_id = $this->EmailGroups->add([
                    'action' => $email['action'],
                    'type' => $email['type'],
                    'plugin_dir' => $email['plugin_dir'],
                    'tags' => $email['tags']
                ]);
            }

            // Set from hostname to use that which is configured for the company
            if (isset(Configure::get('Blesta.company')->hostname)) {
                $email['from'] = str_replace(
                    '@mydomain.com',
                    '@' . Configure::get('Blesta.company')->hostname,
                    $email['from']
                );
            }

            // Add the email template for each language
            foreach ($languages as $language) {
                $this->Emails->add([
                    'email_group_id' => $group_id,
                    'company_id' => Configure::get('Blesta.company_id'),
                    'lang' => $language->code,
                    'from' => $email['from'],
                    'from_name' => $email['from_name'],
                    'subject' => $email['subject'],
                    'text' => $email['text'],
                    'html' => $email['html']
                ]);
            }
        }

		// Add all license_keys tables, *IFF* not already added
		try {
			// license_keys_feeds
			$this->Record->
			setField( 'license_keys_id', [ 'type'           => 'int',
			                               'size'           => 11,
			                               'unsigned'       => true,
			                               'auto_increment' => true
			] )->
			//setField( 'first_name', [ 'type' => 'varchar', 'size' => 64 ] )->
			//setField( 'last_name', [ 'type' => 'varchar', 'size' => 64 ] )->
			//setField( 'email', [ 'type' => 'varchar', 'size' => 64 ] )->
			setField( 'client_id', [ 'type' => 'varchar', 'size' => 32 ] )->
			setField( 'client_id_code', [ 'type' => 'varchar', 'size' => 32 ] )->
			setField( 'extension_name', [ 'type' => 'varchar', 'size' => 256 ] )->
			setField( 'version', [ 'type' => 'varchar', 'size' => 64 ] )->
			setField( 'installation_url', [ 'type' => 'varchar', 'size' => 64 ] )->
			setField( 'ip_address', [ 'type' => 'varchar', 'size' => 64 ] )->
			setField( 'license_code', [ 'type' => 'varchar', 'size' => 64 ] )->
            setField('blesta_license_code', ['type' => 'varchar', 'size' => 64])->
			setField( 'subscribed', [ 'type' => 'char', 'size' => 3 ] )->
			setField( 'created_at', [ 'type' => 'datetime', 'is_null' => true, 'default' => null ] )->
            setField('license_id', ['type' => 'int', 'size' => 11])->
            setField('service_id', ['type' => 'int', 'size' => 11])->
            setField('support_service_id', ['type' => 'int', 'size' => 11])->
            setField('support_service_status', ['type' => 'varchar', 'size' => 64])->
            setField('renewal_date', ['type' => 'datetime', 'is_null' => true, 'default' => null])->
            setField('renewed_date', ['type' => 'datetime', 'is_null' => true, 'default' => null])->
            setKey(['license_keys_id'], 'primary')->
            create('license_keys', true);

		} catch ( Exception $e ) {
			// Error adding... no permission?
			$this->Input->setErrors( [ 'db' => [ 'create' => $e->getMessage() ] ] );

			return;
		}

	}

    /**
     * Performs migration of data from $current_version (the current installed version)
     * to the given file set version
     *
     * @param string $current_version The current installed version of this plugin
     * @param int $plugin_id The ID of the plugin being upgraded
     */
    public function upgrade($current_version, $plugin_id) {
        if (!isset($this->Record)) {
            Loader::loadComponents($this, ['Record']);
        }
        Configure::load('license_keys', dirname(__FILE__) . DS . 'config' . DS);
        Loader::loadModels($this, ['Emails', 'EmailGroups', 'Languages', 'Services', 'Packages']);

        // Upgrade if possible
        if (version_compare($this->getVersion(), $current_version, '>')) {
        // Fetch all currently-installed languages for this company, for which email templates should be created for
            $languages = $this->Languages->getAll(Configure::get('Blesta.company_id'));


            // Add all email templates
            $emails = Configure::get('LicenseKeys.install.emails');
            foreach ($emails as $email) {
                $group = $this->EmailGroups->getByAction($email['action']);
                if ($group) {
                    $group_id = $group->id;
                } else {
                    $group_id = $this->EmailGroups->add([
                        'action' => $email['action'],
                        'type' => $email['type'],
                        'plugin_dir' => $email['plugin_dir'],
                        'tags' => $email['tags']
                    ]);
                    // Set from hostname to use that which is configured for the company
                    if (isset(Configure::get('Blesta.company')->hostname)) {
                        $email['from'] = str_replace(
                            '@mydomain.com',
                            '@' . Configure::get('Blesta.company')->hostname,
                            $email['from']
                        );
                    }

                    // Add the email template for each language
                    foreach ($languages as $language) {
                        $this->Emails->add([
                            'email_group_id' => $group_id,
                            'company_id' => Configure::get('Blesta.company_id'),
                            'lang' => $language->code,
                            'from' => $email['from'],
                            'from_name' => $email['from_name'],
                            'subject' => $email['subject'],
                            'text' => $email['text'],
                            'html' => $email['html']
                        ]);
                    }
                }
            }

             /**
             // use this code not the bottom one
             $this->Record->query(
                "ALTER TABLE `license_keys`
                    ADD `blesta_license_code` VARCHAR( 64 ) NOT NULL DEFAULT '' AFTER `license_code`,
                    ADD `support_service_id` int(11) NOT NULL DEFAULT '' AFTER `service_id`,
                    ADD `converting` char(1) NOT NULL DEFAULT '' AFTER `support_service_id`,
                    ADD `converted` int(11) NOT NULL DEFAULT '' AFTER `converting`,
                    ADD `support_service_status` VARCHAR( 64 ) NOT NULL DEFAULT '' AFTER `converted`,
                    ADD `sticky_notes_id` int(11) AFTER `support_service_status`;"
            );
            
            $this->Record->query(
                "ALTER TABLE `license_keys`
                    ADD `support_service_id` int(11) NOT NULL AFTER `service_id`,
                    ADD `support_service_status` VARCHAR( 64 ) NOT NULL DEFAULT '' AFTER `support_service_id`;"
            );**/

            /**try {
                // license_keys_feeds
                $this->Record->
                setField('license_keys_howto_id', ['type' => 'int',
                    'size' => 11,
                    'unsigned' => true,
                    'auto_increment' => true
                ])->
                setField('client_id', ['type' => 'int', 'size' => 11])->
                setField('wp_user_id', ['type' => 'int', 'size' => 11])->
                setField('status', ['type' => 'char', 'size' => 1])->
                setKey(['license_keys_howto_id'], 'primary')->
                create('license_keys_howto', true);
            } catch (Exception $e) {
                // Error adding... no permission?
                $this->Input->setErrors(['db' => ['create' => $e->getMessage()]]);
                return;
            } **/
        }
    }

	/**
	 * Performs any necessary cleanup actions
	 *
	 * @param int $plugin_id The ID of the plugin being uninstalled
	 * @param bool $last_instance True if $plugin_id is the last instance
	 *  across all companies for this plugin, false otherwise
	 */
	public function uninstall( $plugin_id, $last_instance ) {
		if ( ! isset( $this->Record ) ) {
			Loader::loadComponents( $this, [ 'Record' ] );
        }

        Configure::load('license_keys', dirname(__FILE__) . DS . 'config' . DS);
        Loader::loadModels($this, ['Emails', 'EmailGroups']);
        $emails = Configure::get('LicenseKeys.install.emails');

        // Remove emails and email groups as necessary
        foreach ($emails as $email) {
            // Fetch the email template created by this plugin
            $group = $this->EmailGroups->getByAction($email['action']);

            // Delete all emails templates belonging to this plugin's email group and company
            if ($group) {
                $this->Emails->deleteAll($group->id, Configure::get('Blesta.company_id'));

                if ($last_instance) {
                    $this->EmailGroups->delete($group->id);
                }
            }
        }

		// Remove all license_keys tables *IFF* no other company in the system is using this plugin
		if ( $last_instance ) {
			// $this->Record->drop('license_keys');
		}
	}

	/**
	 * Returns all actions to be configured for this widget
	 * (invoked after install() or upgrade(), overwrites all existing actions)
	 *
	 * @return array A numerically indexed array containing:
	 *  - action The action to register for
	 *  - uri The URI to be invoked for the given action
	 *  - name The name to represent the action (can be language definition)
	 */
	 public function getActions() {
		return [
			[
				'location'  => 'nav_staff',
				'url'     => 'plugin/license_keys/admin_main/index',
				'name'    => Language::_( 'LicenseKeysPlugin.name', true ),
				'options' => [
					'sub' => [
						[
							'url'  => 'plugin/license_keys/admin_main/index',
							'name' => 'LicenseKeysPlugin.list'
						],
                        [
                            'url'  => 'plugin/license_keys/admin_create/index',
                            'name' => 'LicenseKeysPlugin.create'
                        ],
                        [
							'url'  => 'plugin/license_keys/admin_send_emails/index',
							'name' => 'LicenseKeysPlugin.send_emails'
						],
					]
				]
			],
        ];
    }

	/**
	 * Returns all events to be registered for this plugin
	 * (invoked after install() or upgrade(), overwrites all existing events)
	 *
	 * @return array A numerically indexed array containing:
	 *  - event The event to register for
	 *  - callback A string or array representing a callback function or class/method.
	 *      If a user (e.g. non-native PHP) function or class/method, the plugin must
	 *      automatically define it when the plugin is loaded. To invoke an instance
	 *      methods pass "this" instead of the class name as the 1st callback element.
	 */
    public function getEvents() {
        return [
            [
                'event' => 'Services.add',
                'callback' => ['this', 'serviceAdded']
            ],
            [
                'event' => 'Services.edit',
                'callback' => ['this', 'serviceEdited']
            ],
            [
                'event' => 'Services.cancelAfter',
                'callback' => ['this', 'serviceCancelled']
            ],
            [
                'event' => 'Services.suspendAfter',
                'callback' => ['this', 'serviceSuspended']
            ],
            [
                'event' => 'Services.unsuspendAfter',
                'callback' => ['this', 'serviceUnsuspend']
            ],
            [
                //'event' => 'Invoices.setClosedAfter',
                //'callback' => ['this', 'invoicePaid']
            ],
            [
                'event' => 'Clients.createBefore',
                'callback' => ['this', 'createAccountBefore']
            ],
            [
                'event' => 'Clients.createAfter',
                'callback' => ['this', 'createAccountAfter']
            ],
        ];
    }

    public function createAccountBefore($event) {
        if ( ! isset( $this->Record ) ) {
            Loader::loadComponents( $this, [ 'Record','Session' ] );
        }
        $params = $event->getParams();
        $formattedParams = json_decode(json_encode($params), true);
        $email = $formattedParams['vars']['email'];
        $this->Session->write($email.'_password',$formattedParams['vars']['new_password']);
    }

    public function createAccountAfter($event) {
        if ( ! isset( $this->Record ) ) {
            Loader::loadComponents( $this, [ 'Record','Session' ] );
        }

        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList']);

        $params = $event->getParams();
        $formattedParams = json_decode(json_encode($params), true);
        $email = $formattedParams['client']['email'];
        $data['password'] = $this->Session->read($email.'_password');
        $data['username'] = $formattedParams['client']['username'];
        $data['first_name'] = $formattedParams['client']['first_name'];
        $data['last_name'] = $formattedParams['client']['last_name'];
        $data['email'] = $formattedParams['client']['email'];
        $data['roles'] = 'subscriber';
        $vars['client_id'] = $formattedParams['client']['id'];
        $vars['wp_user_id'] = $this->register($data);
        $vars['status'] = '0';
        $this->LicenseKeysList->createWpUser($vars);
        $this->Session->clear($email.'_password');
    }

    /**
     * @param $data
     * @return mixed
     */
    public function register($data) {
        $curl = curl_init();
        $post = [
            "username" => $data["username"],
            "first_name" => $data["first_name"],
            "last_name" => $data["last_name"],
            "email" => $data["email"],
            "password" => $data["password"],
            "roles" => $data["roles"],
            "meta" => [
                "_is_disabled" => "1"
            ]
        ];
        $wp_url = 'https://abdev7.com/wp';
        $app_password = 'yIDfoQDbnIE1WhDg53mr0pEe';
        curl_setopt_array($curl, array(
            CURLOPT_URL => "$wp_url/wp-json/wp/v2/users",
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_USERPWD => "admin:$app_password",
            CURLOPT_POSTFIELDS => json_encode($post),
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));

        $err = curl_error($curl);
        $response = curl_exec($curl);
        $result = json_decode($response,true);

        curl_close($curl);
        return $result["id"];
    }

    /**
     * @param $data
     * @return array|bool|float|int|mixed|stdClass|string|null
     */
    public function updateStatus($data) {
        $curl = curl_init();
        $wp_url = 'https://abdev7.com/wp';
        $app_password = 'yIDfoQDbnIE1WhDg53mr0pEe';
        $post = [
            "id" => $data["wp_user_id"],
            "meta"=> ["_is_disabled" => $data["is_disabled"]]
        ];

        curl_setopt_array($curl, array(
            CURLOPT_URL => "$wp_url/wp-json/wp/v2/users/".$data["wp_user_id"],
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_USERPWD => "admin:$app_password",
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => json_encode($post),
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json"
            ),
        ));

        $response = curl_exec($curl);
        $result = json_decode($response,true);
        curl_close($curl);
        return $result;
    }

    /**
     * Update License Key List Service ID and Renewal Date
     *
     * @param EventObject $event The event to process
     */
    public function serviceAdded($event) {
        if ( ! isset( $this->Record ) ) {
            Loader::loadComponents( $this, [ 'Record' ] );
        }
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Emails', 'Services', 'Packages', 'Pricings', 'ModuleManager']);
        $params = $event->getParams();
        $return = $event->getReturnValue();

        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;

        $formattedParams = json_decode(json_encode($params), true);
        $service_id = $formattedParams['service_id'];

        $service_info = $this->Services->get($service_id);
        $serviceInfo = json_decode(json_encode($service_info), true);

        $parent_service_id = $serviceInfo['parent_service_id'];
        $parentServiceInfo = [];

        if ($parent_service_id !== null) {
            $parent_service_info = $this->Services->get($parent_service_id);
            $parentServiceInfo = json_decode(json_encode($parent_service_info), true);
        }

        $client_id = $serviceInfo['client_id'];
        $client_info = $this->Clients->get($client_id);
        $clientInfo = json_decode(json_encode($client_info), true);
        $clientIdCode = $clientInfo['id_code'];

        $pricing_info = $this->Pricings->get($serviceInfo['pricing_id']);
        $pricingInfo = json_decode(json_encode($pricing_info), true);
        $created_date = $serviceInfo['date_added'];
        $renewal_date = $serviceInfo['date_renews'];

        $package = $this->Packages->getByPricingId($serviceInfo['pricing_id']);
        $package_info = json_decode(json_encode($package), true);
        $packageInfo = $serviceInfo['package'];
        $packageName = $packageInfo['name'];

        $module_info = $this->ModuleManager->initModule($package_info['module_id'], $package_info['company_id']);
        $module_row_info = $this->ModuleManager->getRow($serviceInfo['module_row_id']);
        $module_row = json_decode(json_encode($module_row_info), true);

        // Set all acceptable module meta fields
        $module_fields = [];
        if (!empty($module_row['meta'])) {
            $tags = $module_info->getEmailTags();
            $tags = (isset($tags['module']) && is_array($tags['module']) ? $tags['module'] : []);

            if (!empty($tags)) {
                foreach ($module_row['meta'] as $key => $value) {
                    if (in_array($key, $tags)) {
                        $module_fields[$key] = $value;
                    }
                }
            }
        }
        $module = $module_fields;

        $status = $serviceInfo['status'];
        $new_service = $serviceInfo['date_last_renewed'] === null ? true : false;

        $getTz = $this->Companies->getSetting(1, 'timezone');
        $tz = json_decode(json_encode($getTz), true);
        $timezone = $tz['value'];
        if ($renewal_date === $created_date) {
            $renewal_date = null;
        }

        $services = ['Basic','Business','Business Plus','Dropshipper'];

        if (!in_array($packageInfo['name'],$services)) {

            $search = ['Buy', 'Rent', ' License'];
            $replace = [''];
            $extensionName = str_replace($search, $replace, $packageInfo['name']);
            //$servicesList = $this->Services->getList($client_id, 'active', 1, array(), false, array('package_name' => $extensionName));
            //$services = json_decode(json_encode($servicesList), true);

            $existingLicenseInfo = $this->LicenseKeysList->getLicensesByClientId($client_id, $extensionName);
            $services = json_decode(json_encode($existingLicenseInfo), true);

            $existing_license_key_id = $services['license_keys_id'];
            $existing_service_id = $services['service_id'];

            $service_added_date = $renewing_date = $action = '';
            if ($status === 'active') {
                if ($new_service && $existing_license_key_id) {
                    $action = 'LicenseKeys.convert_license';
                } else {
                    $action = 'LicenseKeys.service_added';
                }
                $license_key = "";

                if (sizeof($parentServiceInfo) > 0) {
                    foreach ($parentServiceInfo['fields'] as $parentService) {
                        if ($parentService['key'] !== 'license_module_key') {
                            continue;
                        }
                        if ($parentService['key'] === 'license_module_key' && !is_null($parentService['value'])) {
                            $license_key = $parentService['value'];
                        }
                    }
                } else {
                    foreach ($serviceInfo['fields'] as $service) {
                        if ($service['key'] !== 'license_module_key') {
                            continue;
                        }
                        if ($service['key'] === 'license_module_key' && !is_null($service['value'])) {
                            $license_key = $service['value'];
                        }
                    }
                }

                if ($existing_license_key_id) {
                    $vars = array(
                        'blesta_license_code' => $license_key,
                        'service_id' => $service_id,
                        'support_service_id' => 0,
                        'support_service_status' => 'Active',
                        'renewal_date' => $renewal_date
                    );
                    $this->LicenseKeysList->update($existing_license_key_id, $vars);

                    $service_cancel_date = new DateTime('now', new DateTimeZone($timezone));
                    $service_cancel_date->setTimezone(new DateTimeZone($timezone));
                    $currentDate = $service_cancel_date->modify('-1 day')->format('M d, Y');
                    $serviceCancelVars = [
                        'date_canceled' => $currentDate,
                        'use_module' => false,
                        'reapply_payments' => false,
                        'notify_cancel' => false,
                        'cancellation_reason' => 'Service is replaced by license product.'
                    ];
                    $this->Services->cancel(
                        $existing_service_id,
                        $serviceCancelVars
                    );
                }

                $service_email_content = null;

                foreach ($packageInfo['email_content'] as $index => $email) {

                    // Save English so we can use it if the default language is not available
                    if ($email['lang'] == 'en_us') {
                        $service_email_content = $email;
                    }

                    // Use the client's default language
                    if ($clientInfo['settings']['language'] == $email['lang']) {
                        $service_email_content = $email;
                        break;
                    }
                }

                $actions = [
                    'LicenseKeys.service_added',
                    'LicenseKeys.convert_license',
                ];

                $search_tags = [
                    '{client.id}',
                    '{client.id_code}',
                    '{client.first_name}',
                    '{client.last_name}',
                    '{pricing.term}',
                    '{pricing.period}',
                    '{pricing.currency}',
                    '{pricing.price}',
                    '{pricing.setup_fee}',
                    '{pricing.cancel_fee}',
                    '{module.name}',
                    '{module.label}',
                    '{package.name}',
                    '{service.license_module_key}'
                ];
                $replace_tags = [
                    $clientInfo['id'],
                    $clientInfo['id_code'],
                    $clientInfo['first_name'],
                    $clientInfo['last_name'],
                    $pricingInfo['term'],
                    $pricingInfo['period'],
                    $pricingInfo['currency'],
                    $pricingInfo['price'],
                    $pricingInfo['setup_fee'],
                    $pricingInfo['cancel_fee'],
                    isset($module['name']) ? $module['name'] : '',
                    isset($module['label']) ? $module['label'] : '',
                    $package_info['name'],
                    $license_key
                ];

                $email_html = $email_text = '';
                if (in_array($action, $actions)) {
                    if (isset($service_email_content['html'])) {
                        $email_html = str_replace(
                            $search_tags,
                            $replace_tags,
                            $service_email_content['html']
                        );
                    }

                    if (isset($service_email_content['text'])) {
                        $email_text = str_replace(
                            $search_tags,
                            $replace_tags,
                            $service_email_content['text']
                        );
                    }
                }

                $tags = ['contact' => $client_info,
                    'service' => $service_info,
                    'package' => $packageInfo,
                    'license_key' => $license_key,
                    'package_email_html' => $email_html,
                    'package_email_text' => $email_text,
                    'created_date' => $service_added_date,
                    'renewal_date' => $renewing_date
                ];

                //$this->logger->error('service added action: '.$action);
                //if ($action!=='') {
                $this->Emails->send(
                    $action,
                    Configure::get('Blesta.company_id'),
                    $clientInfo['settings']['language'],
                    $clientInfo['email'],
                    $tags
                );
                //}
            }
        }
    }

    /**
     * Update License Key and License Key List
     *
     * @param EventObject $event The event to process
     */
     public function serviceEdited($event) {
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Emails', 'Services', 'Packages', 'Pricings', 'ModuleManager']);
        $params = $event->getParams();
        $return = $event->getReturnValue();
        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;
        $formattedParams = json_decode(json_encode($params), true);

        $service_id = $formattedParams['service_id'];
        $new_license_key = $existing_license_key = '';
        if (isset($formattedParams['vars']['license_module_key'])) {
            $new_license_key = $formattedParams['vars']['license_module_key'];
        }

        $service_info = $this->Services->get($service_id);
        $serviceInfo = json_decode(json_encode($service_info), true);
        $parent_service_id = $serviceInfo['parent_service_id'];
        $parentServiceInfo = [];

        if ($parent_service_id !== null) {
            $parent_service_info = $this->Services->get($parent_service_id);
            $parentServiceInfo = json_decode(json_encode($parent_service_info), true);
        }

        $client_id = $serviceInfo['client_id'];
        $client_info = $this->Clients->get($client_id);
        $clientInfo = json_decode(json_encode($client_info), true);
        $clientIdCode = $clientInfo['id_code'];

        $pricing_info = $this->Pricings->get($serviceInfo['pricing_id']);
        $pricingInfo = json_decode(json_encode($pricing_info), true);
        $created_date = $serviceInfo['date_added'];
        $renewal_date = $serviceInfo['date_renews'];

        $package = $this->Packages->getByPricingId($serviceInfo['pricing_id']);
        $package_info = json_decode(json_encode($package), true);
        $packageInfo = $serviceInfo['package'];
        $packageName = $packageInfo['name'];

        $module_info = $this->ModuleManager->initModule($package_info['module_id'], $package_info['company_id']);
        $module_row_info = $this->ModuleManager->getRow($serviceInfo['module_row_id']);
        $module_row = json_decode(json_encode($module_row_info), true);

        // Set all acceptable module meta fields
        $module_fields = [];
        if (!empty($module_row['meta'])) {
            $tags = $module_info->getEmailTags();
            $tags = (isset($tags['module']) && is_array($tags['module']) ? $tags['module'] : []);

            if (!empty($tags)) {
                foreach ($module_row['meta'] as $key => $value) {
                    if (in_array($key, $tags)) {
                        $module_fields[$key] = $value;
                    }
                }
            }
        }
        $module = $module_fields;
        //
        $status = $serviceInfo['status'];
        $new_service = $serviceInfo['date_last_renewed'] === null ? true : false;

        $getTz = $this->Companies->getSetting(1, 'timezone');
        $tz = json_decode(json_encode($getTz), true);
        $timezone = $tz['value'];
        if ($renewal_date === $created_date) {
            $renewal_date = null;
        }

        $services = ['Business','Business Plus','Dropshipper'];

        if ($status === 'active' && $new_service && in_array($packageInfo['name'],$services)) {
            $wp_user_id = json_decode(json_encode($this->LicenseKeysList->getWpUserID($formattedParams['vars']['client_id'])),true);
            $data['wp_user_id'] = $wp_user_id['wp_user_id'];
            $this->updateStatus($data);
        }

        $search = ['Buy ', 'Rent ', ' Support And Updates Service'];
        $replace = [''];
        $extensionName = str_replace($search, $replace, $packageInfo['name']);

        $license_key = $service_added_date = $renewing_date = "";
        if (sizeof($parentServiceInfo) > 0) {
            foreach ($parentServiceInfo['fields'] as $parentService) {
                if ($parentService['key'] !== 'license_module_key') {
                    continue;
                }
                if ($parentService['key'] === 'license_module_key' && !is_null($parentService['value'])) {
                    $license_key = $parentService['value'];
                }
            }
        } else {
            foreach ($serviceInfo['fields'] as $service) {
                if ($service['key'] !== 'license_module_key') {
                    continue;
                }
                if ($service['key'] === 'license_module_key' && !is_null($service['value'])) {
                    $license_key = $service['value'];
                }
            }
        }

        if ($status === 'active' && $new_service) {
            if ($license_key !== $new_license_key) {
                //$this->sendData(['service_id' => $service_id, 'license_code' => $license_key, 'license_status' => 'Active']);
                $license_list_info = $this->LicenseKeysList->getLicenseKeysId($service_id);
            }

            $action = 'LicenseKeys.service_added';

            $service_email_content = null;
            foreach ($packageInfo['email_content'] as $index => $email) {
                if ($license_key) {
                    $email['html'] .= "<p>Your license key can always be found within your AbanteCart Plus account dashboard.</p>";
                }

                // Save English so we can use it if the default language is not available
                if ($email['lang'] == 'en_us') {
                    $service_email_content = $email;
                }

                // Use the client's default language
                if ($clientInfo['settings']['language'] == $email['lang']) {
                    $service_email_content = $email;
                    break;
                }
            }

            $actions = [
                'LicenseKeys.service_added',
            ];

            $search_tags = [
                '{client.id}',
                '{client.id_code}',
                '{client.first_name}',
                '{client.last_name}',
                '{pricing.term}',
                '{pricing.period}',
                '{pricing.currency}',
                '{pricing.price}',
                '{pricing.setup_fee}',
                '{pricing.cancel_fee}',
                '{module.name}',
                '{module.label}',
                '{package.name}',
                '{service.license_module_key}'
            ];
            $replace_tags = [
                $clientInfo['id'],
                $clientInfo['id_code'],
                $clientInfo['first_name'],
                $clientInfo['last_name'],
                $pricingInfo['term'],
                $pricingInfo['period'],
                $pricingInfo['currency'],
                $pricingInfo['price'],
                $pricingInfo['setup_fee'],
                $pricingInfo['cancel_fee'],
                isset($module['name']) ? $module['name'] : '',
                isset($module['label']) ? $module['label'] : '',
                $package_info['name'],
                $license_key
            ];

            $email_html = $email_text = '';
            if (in_array($action, $actions)) {
                if (isset($service_email_content['html'])) {
                    $email_html = str_replace(
                        $search_tags,
                        $replace_tags,
                        $service_email_content['html']
                    );
                }

                if (isset($service_email_content['text'])) {
                    $email_text = str_replace(
                        $search_tags,
                        $replace_tags,
                        $service_email_content['text']
                    );
                }
            }

            $tags = [
                'contact' => $client_info,
                'service' => $service_info,
                'package' => $packageInfo,
                'license_key' => $license_key,
                'package_email_html' => $action === 'LicenseKeys.service_added_to_existing' ? '' : $email_html,
                'package_email_text' => $action === 'LicenseKeys.service_added_to_existing' ? '' : $email_text,
                'created_date' => $service_added_date,
                'renewal_date' => $renewing_date
            ];

            $this->Emails->send(
                $action,
                Configure::get('Blesta.company_id'),
                $clientInfo['settings']['language'],
                $clientInfo['email'],
                $tags
            );
        }
    }

    /**
     * Cancel Service
     *
     * @param EventObject $event The event to process
     */
    public function serviceCancelled($event) {
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Contacts', 'Services', 'Emails']);
        $params = $event->getParams();
        $return = $event->getReturnValue();
        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;
        $action = '';
        $formattedParams = json_decode(json_encode($params), true);
        $service_id = $formattedParams['service_id'];
        $client_id = $formattedParams['old_service']['client_id'];
        $parent_service_id = $formattedParams['old_service']['parent_service_id'];
        $client_info = $this->Clients->get($client_id);
        $clientInfo = json_decode(json_encode($client_info), true);
        $clientIdCode = $clientInfo['id_code'];
        $contact_id = $clientInfo['contact_id'];
        $contact = $this->Contacts->get($contact_id);
        $service_info = $this->Services->get($service_id);
        $serviceInfo = json_decode(json_encode($service_info), true);
        $packageInfo = $serviceInfo['package'];

        //$tags = ['contact' => $contact, 'package' => $packageInfo, 'service' => $service_info];

        $extensionName = $packageInfo['name'];

        $services = ['Business','Business Plus','Dropshipper'];

        if (in_array($extensionName,$services)) {
            $wp_user_id = json_decode(json_encode($this->LicenseKeysList->getWpUserID($client_id)),true);
            $data['wp_user_id'] = $wp_user_id['wp_user_id'];
            $data['is_disabled'] = '1';
            $this->updateStatus($data);
        }

        /**if (stripos($extensionName, 'Additional ') == false && stripos($extensionName, ' Development') == false
            && stripos($extensionName, ' Support And Updates Service') == false && stripos($extensionName, 'Rent ') == false
            && stripos($extensionName, ' Development') == false) {
            $this->LicenseKeysList->deleteByServiceId($service_id);
        } elseif (stripos($extensionName, ' Support And Updates Service') !== false && !is_null($parent_service_id)) {
            $action = 'LicenseKeys.support_service_canceled';
            $this->LicenseKeysList->updateSupportStatus($service_id,'Cancelled');
        } elseif (stripos($extensionName, 'Rent ') !== false) {
            $action = 'LicenseKeys.rent_service_canceled';
        }

        $this->sendData(['service_id'=>$service_id,'license_status'=>'Cancelled']);

        $this->Emails->send(
            $action,
            Configure::get('Blesta.company_id'),
            $clientInfo['settings']['language'],
            $clientInfo['email'],
            $tags
        ); **/
    }

    public function serviceSuspended($event) {
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Contacts', 'Services', 'Emails']);
        $params = $event->getParams();
        $return = $event->getReturnValue();
        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;
        $action = '';
        $formattedParams = json_decode(json_encode($params), true);
        $service_id = $formattedParams['service_id'];
        $client_id = $formattedParams['old_service']['client_id'];

        $service_info = $this->Services->get($service_id);
        $serviceInfo = json_decode(json_encode($service_info), true);
        $packageInfo = $serviceInfo['package'];

        //$tags = ['contact' => $contact, 'package' => $packageInfo, 'service' => $service_info];

        $extensionName = $packageInfo['name'];

        $services = ['Business', 'Business Plus', 'Dropshipper'];

        if (in_array($extensionName, $services)) {
            $wp_user_id = json_decode(json_encode($this->LicenseKeysList->getWpUserID($client_id)), true);
            $data['wp_user_id'] = $wp_user_id['wp_user_id'];
            $data['is_disabled'] = '1';
            $this->updateStatus($data);
        }
    }

    public function serviceUnsuspend($event) {
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Contacts', 'Services', 'Emails']);
        $params = $event->getParams();
        $return = $event->getReturnValue();
        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;
        $action = '';
        $formattedParams = json_decode(json_encode($params), true);
        $service_id = $formattedParams['service_id'];
        $client_id = $formattedParams['old_service']['client_id'];

        $service_info = $this->Services->get($service_id);
        $serviceInfo = json_decode(json_encode($service_info), true);
        $packageInfo = $serviceInfo['package'];

        //$tags = ['contact' => $contact, 'package' => $packageInfo, 'service' => $service_info];

        $extensionName = $packageInfo['name'];

        $services = ['Business', 'Business Plus', 'Dropshipper'];

        if (in_array($extensionName, $services)) {
            $wp_user_id = json_decode(json_encode($this->LicenseKeysList->getWpUserID($client_id)), true);
            $data['wp_user_id'] = $wp_user_id['wp_user_id'];
            $data['is_disabled'] = '0';
            $this->updateStatus($data);
        }
    }

    /**
     * Update Renewal Date
     *
     * @param EventObject $event The event to process
     */
    /** public function invoicePaid($event) {
        Loader::loadModels($this, ['LicenseKeys.LicenseKeysList', 'Companies', 'Clients', 'Services', 'Emails']);
        $params = $event->getParams();
        $return = $event->getReturnValue();

        $logger = $this->getFromContainer('logger');
        $this->logger = $logger;

        $formattedParams = json_decode(json_encode($params), true);
        $invoice_id = $formattedParams['invoice_id'];
        $this->sendData(['invoice_id'=>$invoice_id]);
        $lineItems = $formattedParams['old_invoice']['line_items'];
        $client_id = $formattedParams['old_invoice']['client_id'];
        $client_info = $this->Clients->get($client_id);
        $clientInfo = json_decode(json_encode($client_info), true);
        //$message = 'Thank you for renewing your extension(s) support time and updates. We have extended: <br>';
        $additional_message = '';
        $items = $rent_items = 0;
        $message = $renting_message = $action = $tags = '';
        foreach ($lineItems as $items) {
            $service_id = $items['service_id'];
            if (is_null($service_id)) {
                continue;
            }

            $servicesInfo = json_decode(json_encode($this->Services->get($service_id)), true);
            $renewal_date = $servicesInfo['date_renews'];
            $new_service = $servicesInfo['date_last_renewed'] === null ? true : false;
            //$this->logger->error('service id: '.$service_id.', description: '.$items['description'].', new service:'.$new_service);
            if (!$new_service) {
                $getTz = $this->Companies->getSetting(1, 'timezone');
                $tz = json_decode(json_encode($getTz), true);
                $timezone = $tz['value'];

                $renewed_date = date_create($renewal_date, new DateTimeZone($timezone))->format('Y-m-d');
                $renewedDate = date_create($renewal_date, new DateTimeZone($timezone))->format('M d, Y');
                if (is_null($formattedParams['old_invoice']['date_closed'])) {
                    $pd_date = '';
                } else {
                    $pd_date = $formattedParams['old_invoice']['date_closed'];
                }
                $paid_date = date_create($pd_date, new DateTimeZone($timezone))->format('Y-m-d');

                if (stripos($items['description'], ' Support And Updates Service') !== false) {
                    $items++;
                    $items['description'] = str_replace(' Support And Updates Service', '', $items['description']);
                    if (stripos($items['description'], 'Prorated') !== false) {
                        $description = str_replace('Prorated ', '', $items['description']);
                        $description = explode(' (', $description);
                    } else {
                        $description = explode(' - ', $items['description']);
                    }

                    $this->LicenseKeysList->updateSupportRenewDate($description[0], $service_id, $renewed_date, $paid_date);
                    $message .= '<strong>' . $description[0] . '</strong> support time and updates to <strong>' . $renewedDate . '</strong>.<br>';
                }
                if (stripos($items['description'], 'Rent ') !== false) {
                    $rent_items++;
                    if (stripos($items['description'], 'Prorated') !== false) {
                        $description = str_replace('Prorated ', '', $items['description']);
                        $description = explode(' (', $description);
                    } else {
                        $description = $items['description'];
                    }

                    $this->LicenseKeysList->updateRenewDate($description, $service_id, $renewed_date, $paid_date);
                    $renting_message .= '<strong>' . $description . '</strong> renting time to <strong>' . $renewedDate . '</strong>.<br>';
                }
            }
        }

        if ($items > 0 && $message !== '') {
            $tags = ['packages' => $message];
            $action = 'LicenseKeys.support_service_paid';
            $this->Emails->send(
                $action,
                Configure::get('Blesta.company_id'),
                $clientInfo['settings']['language'],
                $clientInfo['email'],
                $tags
            );
        }

        if ($rent_items > 0 && $renting_message !== '') {
            $tags = ['packages' => $renting_message];
            $action = 'LicenseKeys.rent_service_paid';
            $this->Emails->send(
                $action,
                Configure::get('Blesta.company_id'),
                $clientInfo['settings']['language'],
                $clientInfo['email'],
                $tags
            );
        }
    } **/

    /**
     * @param $data
     * @return void
     */
    /** private function sendData($data) {
        $url="https://abdev7.com/store/index.php?rt=license_generator/webhooks/";

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, $url);
        curl_setopt($curl, CURLOPT_POST, 1);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        $response = curl_exec($curl);
    } **/

}
