<?php
/**
 * License Keys List
 *
 * Manages license keys
 *
 * @package blesta
 * @subpackage blesta.plugins.license_keys.models
 * @copyright Copyright (c) 2020, We Hear You 2, Inc.
 */
use Blesta\Core\Util\Common\Traits\Container;


class LicenseKeysList extends LicenseKeysModel {

    use Container;

    /**
     * Initialize Feeds
     */
    public function __construct() {
        parent::__construct();
        Language::loadLang('license_keys_list', null, PLUGINDIR . 'license_keys' . DS . 'language' . DS);
    }

    /**
     * Returns a list of license keys
     *
     *  (if not given will fetch details for ALL license keys)
     * @param int $page The page number to fetch infos for
     * @param array $order The order to sort the result by
     * @param array $filter The search filter
     * @return array An array of stdClass objects each representing an article containing the following:
     *  - firstname First Name
     *  - lastname Last Name
     *  - email Email
     *  - id_code Client ID
     *  - extension_name Extension Name
     *  - version Extension Version
     *  - installation_url Installation URL
     *  - ip_address IP Address
     *  - license_code License Code
     *  - created_at The date the license keys was added
     */
    public function getList( $page = 1, $order = ['created_at' => 'desc'],$filter) {
        $this->Record = $this->getLicenseKeys($filter);
        $list = $this->Record->order($order)
		            ->limit( $this->getPerPage(), ( max( 1, $page ) - 1 ) * $this->getPerPage() )
		            ->fetchAll();

        return $list;
    }

    /**
     * Returns all infos
     *
     *  (if not given will fetch infos for ALL extensions)
     * @return int The number of results
     */
    public function getListCount($filter) {
        $this->Record = $this->getLicenseKeys($filter);
        return $this->Record->numResults();
    }

	/**
	 * Get single license keys
	 *
	 * @return Record A Record object representing the query for fetching a single license keys
	 */
	public function getLicenseKeyById($license_keys_id) {
		return $this->Record->select(array("contacts.first_name",
			"contacts.last_name",
			"contacts.email",
			"contacts.company",
			"states.name"=>"state",
			"countries.name"=>"country",
			"license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)
		                    ->on("states.country_alpha2","=","contacts.country",false)->
			leftJoin("states","states.code","=","contacts.state",false)->leftJoin("countries","countries.alpha2","=","contacts.country", false)->where('license_keys_id', '=', $license_keys_id)->fetch();
	}

    /**
     * Builds a partially constructed query Record object use to fetch or count results
     *  (if not given will fetch list of license keys)
     * @return Record A partially constructed query Record object
     */
    public function getLicenseKeys($filter = array(),$order=null) {
        //if ($filter['client_id_code']!=='') {
        //    $this->Record->select(array("contacts.first_name",
        //        "contacts.last_name",
        //        "contacts.email",
        //        "contacts.company",
        //        "states.name"=>"state",
        //        "countries.name"=>"country",
        //        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
        //    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
        //    leftJoin("states","states.code","=","contacts.state", false)->
        //    like( 'client_id_code', "%" . $filter['client_id_code'] . "%" )->where("contacts.user_id","=",null)->where('contact_type','=','primary');
        //} elseif ($filter['client_id']!=='') {
        //    $this->Record->select(array("contacts.first_name",
        //        "contacts.last_name",
        //        "contacts.email",
        //        "contacts.company",
        //        "states.name"=>"state",
        //        "countries.name"=>"country",
        //        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
        //    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
        //    leftJoin("states","states.code","=","contacts.state", false)->
        //    like( 'license_keys.client_id', "%" . $filter['client_id'] . "%" )->where("contacts.user_id","=",null)->where('contact_type','=','primary');
        //} else
            if ($filter['license_key']!=='') {
            $this->Record->select(array("contacts.first_name",
                "contacts.last_name",
                "contacts.email",
                "contacts.company",
                "states.name"=>"state",
                "countries.name"=>"country",
                "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
            leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
            leftJoin("states","states.code","=","contacts.state", false)->
            like( 'license_code', "%" . $filter['license_key'] . "%" )->where("contacts.user_id","=",null)->where('contact_type','=','primary');
        } elseif ($filter['blesta_license_key']!=='') {
            $this->Record->select(array("contacts.first_name",
                "contacts.last_name",
                "contacts.email",
                "contacts.company",
                "states.name"=>"state",
                "countries.name"=>"country",
                "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
            leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
            leftJoin("states","states.code","=","contacts.state", false)->
            like( 'blesta_license_code', "%" . $filter['blesta_license_key'] . "%" )->where("contacts.user_id","=",null)->where('contact_type','=','primary');
        } elseif ($filter['extension_name']!=='' && $filter['subscribed']!=='') {
		    if ( $filter['from'] !== '' && $filter['to'] !== '' ) {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states","states.code","=","contacts.state", false)->
                    like( 'extension_name', "%" . $filter['extension_name'] . "%" )->where("subscribed", '=', $filter['subscribed'])->where("contacts.user_id","=",null)->where('DATE(created_at)','>=',$filter['from'])->
                    where( 'DATE(created_at)', '<=', $filter['to'] )->where('contact_type','=','primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states","states.code","=","contacts.state", false)->
                    like( 'extension_name', "%" . $filter['extension_name'] . "%" )->where("subscribed", '=', $filter['subscribed'])->where("contacts.user_id","=",null)->where('DATE(created_at)','>=',$filter['from'])->
                    where( 'DATE(created_at)', '<=', $filter['to'] )->where('contact_type','=','primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name" => "state",
                        "countries.name" => "country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("subscribed", '=', $filter['subscribed'])->where("contacts.user_id", "=", null)->where('DATE(created_at)', '>=', $filter['from'])->
                    where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary');
                }
		    } else {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("subscribed", "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("subscribed", "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name" => "state",
                        "countries.name" => "country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("subscribed", "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary');
                }
		    }
	    } elseif ($filter['extension_name']!=='' && $filter['subscribed']=='') {
		    if ( $filter['from'] !== '' && $filter['to'] !== '' ) {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states","states.code","=","contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where('DATE(created_at)', '>=', $filter['from'])->where("contacts.user_id", "=", null)->
                    where( 'DATE(created_at)', '<=', $filter['to'] )->where('contact_type','=','primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from( 'license_keys' )->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries","countries.alpha2","=","contacts.country", false)->on("states.country_alpha2","=","contacts.country",false)->
                    leftJoin("states","states.code","=","contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where('DATE(created_at)', '>=', $filter['from'])->where("contacts.user_id", "=", null)->
                    where( 'DATE(created_at)', '<=', $filter['to'] )->where('contact_type','=','primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    if ($filter['client_id_code']!=='') {
                        $this->Record->select(array("contacts.first_name",
                            "contacts.last_name",
                            "contacts.email",
                            "contacts.company",
                            "states.name"=>"state",
                            "countries.name"=>"country",
                            "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                        leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                        leftJoin("states", "states.code", "=", "contacts.state", false)->
                        like('extension_name', "%" . $filter['extension_name'] . "%")->where('DATE(created_at)', '>=', $filter['from'])->where("contacts.user_id", "=", null)->
                        where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                    } elseif ($filter['client_id']!=='') {
                        $this->Record->select(array("contacts.first_name",
                            "contacts.last_name",
                            "contacts.email",
                            "contacts.company",
                            "states.name"=>"state",
                            "countries.name"=>"country",
                            "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                        leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                        leftJoin("states", "states.code", "=", "contacts.state", false)->
                        like('extension_name', "%" . $filter['extension_name'] . "%")->where('DATE(created_at)', '>=', $filter['from'])->where("contacts.user_id", "=", null)->
                        where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                    } else {
                        $this->Record->select(array("contacts.first_name",
                            "contacts.last_name",
                            "contacts.email",
                            "contacts.company",
                            "states.name" => "state",
                            "countries.name" => "country",
                            "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                        leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                        leftJoin("states", "states.code", "=", "contacts.state", false)->
                        like('extension_name', "%" . $filter['extension_name'] . "%")->where('DATE(created_at)', '>=', $filter['from'])->where("contacts.user_id", "=", null)->
                        where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary');
                    }
                }
		    } else {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name" => "state",
                        "countries.name" => "country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary');
                }
		    }
	    } elseif ($filter['extension_name']==='' && $filter['subscribed']!=='') {
		    if ( $filter['from'] !== '' && $filter['to'] !== '' ) {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where('DATE(created_at)', '>=', $filter['from'])->where('DATE(created_at)', '<=', $filter['to'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where('DATE(created_at)', '>=', $filter['from'])->where('DATE(created_at)', '<=', $filter['to'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name" => "state",
                        "countries.name" => "country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where('DATE(created_at)', '>=', $filter['from'])->where('DATE(created_at)', '<=', $filter['to'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary');
                }
		    } else {
                if ($filter['client_id_code']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
                } elseif ($filter['client_id']!=='') {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name"=>"state",
                        "countries.name"=>"country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
                } else {
                    $this->Record->select(array("contacts.first_name",
                        "contacts.last_name",
                        "contacts.email",
                        "contacts.company",
                        "states.name" => "state",
                        "countries.name" => "country",
                        "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                    leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->
                    where('subscribed', "=", $filter['subscribed'])->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary');
                }
		    }
	    } elseif ($filter['extension_name']=='' && $filter['subscribed']==='' && $filter['from']!=='' && $filter['to']!=='') {
            if ($filter['client_id_code']!=='') {
                $this->Record->select(array("contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name"=>"state",
                    "countries.name"=>"country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                leftJoin("states", "states.code", "=", "contacts.state", false)->like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('DATE(created_at)', '>=', $filter['from'])->
                where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
            } elseif ($filter['client_id']!=='') {
                $this->Record->select(array("contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name"=>"state",
                    "countries.name"=>"country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                leftJoin("states", "states.code", "=", "contacts.state", false)->like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('DATE(created_at)', '>=', $filter['from'])->
                where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
            } else {
                $this->Record->select(array("contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name" => "state",
                    "countries.name" => "country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)->
                leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->on("states.country_alpha2", "=", "contacts.country", false)->
                leftJoin("states", "states.code", "=", "contacts.state", false)->like('extension_name', "%" . $filter['extension_name'] . "%")->where("contacts.user_id", "=", null)->where('DATE(created_at)', '>=', $filter['from'])->
                where('DATE(created_at)', '<=', $filter['to'])->where('contact_type', '=', 'primary');
            }
	    } else {
            if ($filter['client_id_code']!=='') {
                $this->Record->select(array("contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name"=>"state",
                    "countries.name"=>"country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)
                    ->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id_code', "=" , $filter['client_id_code'] );
            } elseif ($filter['client_id']!=='') {
                $logger = $this->getFromContainer('logger');
                $this->logger = $logger;
                $this->Record->select(array("contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name"=>"state",
                    "countries.name"=>"country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)
                    ->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary')->where('license_keys.client_id', "=" , $filter['client_id'] );
            } else {
                $this->Record->select(array(
                    "contacts.first_name",
                    "contacts.last_name",
                    "contacts.email",
                    "contacts.company",
                    "states.name" => "state",
                    "countries.name" => "country",
                    "license_keys.*"))->from('license_keys')->leftJoin("contacts", "contacts.client_id", "=", "license_keys.client_id", false)
                    ->on("states.country_alpha2", "=", "contacts.country", false)->
                    leftJoin("states", "states.code", "=", "contacts.state", false)->leftJoin("countries", "countries.alpha2", "=", "contacts.country", false)->where("contacts.user_id", "=", null)->where('contact_type', '=', 'primary');
            }
        }
	    if ($order) {
			return $this->Record->order($order)->fetchAll();
		} else {
			return $this->Record;
		}

    }

	/**
	 * @param string $extension_name
	 * @param string $from
	 * @param string $to
	 *
	 * @return mixed
	 */
    public function getEmailsByExtensionName($extension_name,$from=NULL,$to=NULL) {
	    if ($extension_name==="all") {
		    if ($from!=='') {
			    return $this->Record->query( "SELECT DISTINCT c.first_name, c.last_name, c.email from license_keys lk
 										 LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL  
 										 WHERE lk.subscribed='Yes' AND c.contact_type='primary' AND DATE(lk.created_at)>=? AND DATE(lk.created_at)<=?",$from,$to )->fetchAll();
		    } else {
			    return $this->Record->query( "SELECT DISTINCT c.first_name, c.last_name, c.email from license_keys lk
 										 LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL  
 										 WHERE lk.subscribed='Yes'AND c.contact_type='primary' " )->fetchAll();
		    }
	    } else {
		    if ($from!='') {
			    return $this->Record->query( "SELECT DISTINCT c.first_name, c.last_name, c.email from license_keys lk 
                                         LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL
                                         WHERE lk.extension_name=? AND c.contact_type='primary' AND lk.subscribed=? AND DATE(lk.created_at)>=? AND DATE(lk.created_at)<=?", $extension_name, 'Yes',$from,$to )->fetchAll();
		    } else {
			    return $this->Record->query( "SELECT DISTINCT c.first_name, c.last_name, c.email from license_keys lk 
                                         LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL
                                         WHERE lk.extension_name=? AND c.contact_type='primary' AND lk.subscribed=?", $extension_name, 'Yes' )->fetchAll();
		    }
	    }

    }

	/**
	 * @param string $extension_name
	 * @param string $from
	 * @param string $to
	 *
	 * @return mixed
	 */
    public function getEmailTotals($extension_name,$from=NULL,$to=NULL) {
		if ($extension_name==="all") {
			 if ($from!=='') {
				 $query = $this->Record->query( "SELECT COUNT(DISTINCT c.email) as total from license_keys lk 
                                           LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL  
                                           WHERE lk.subscribed='Yes' AND c.contact_type='primary' AND DATE(lk.created_at)>=? AND DATE(lk.created_at)<=?",$from,$to )->fetchAll();
			 } else {
				 $query = $this->Record->query( "SELECT COUNT(DISTINCT c.email) as total from license_keys lk 
                                           LEFT JOIN contacts c ON c.client_id = lk.client_id  
                                           WHERE lk.subscribed='Yes' AND c.contact_type='primary'" )->fetchAll();
			 }
			return $query[0]->total;
		} else {
			 if ($from!=='') {
				 $query = $this->Record->query( "SELECT COUNT(DISTINCT c.email) as total from license_keys lk 
                                           LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL  
                                            WHERE lk.extension_name=? AND c.contact_type='primary' AND lk.subscribed=? AND DATE(lk.created_at)>=? AND DATE(lk.created_at)<=?", $extension_name, 'Yes',$from,$to )->fetchAll();
			 } else {
				 $query = $this->Record->query( "SELECT COUNT(DISTINCT c.email) as total from license_keys lk 
                                           LEFT JOIN contacts c ON c.client_id = lk.client_id AND c.user_id is NULL  
                                           WHERE lk.extension_name=? AND c.contact_type='primary' AND lk.subscribed=?", $extension_name, 'Yes' )->fetchAll();
			 }
			return $query[0]->total;
		}
    }

	/**
	 * @param $user_id
	 *
	 * @return mixed
	 */
    public function getClientId($user_id) {
	    return $this->Record->query( "SELECT c.user_id from contacts ct LEFT JOIN clients c ON c.id=ct.client_id where ct.user_id=?", $user_id)->fetch();
    }

	/**
	 * @param array $vars
	 *
	 * @return mixed
	 */
	public function create(array $vars) {
        // Add a client
		//$fields = ['first_name', 'last_name', 'email', 'client_id', 'client_id_code','extension_name','version','installation_url','ip_address','license_code','subscribed','created_at'];
        $fields = ['client_id', 'client_id_code','extension_name','version','installation_url','ip_address','license_code','blesta_license_code','subscribed','created_at','license_id','service_id','support_service_id','support_service_status','renewal_date','renewed_date'];
        $this->Record->insert('license_keys', $vars, $fields);
		$info_id = $this->Record->lastInsertId();
		return $info_id;
	}

	/**
	 * Edit license key from the license keys table
	 *
	 * @param int $license_keys_id The license keys id ID
	 * @param array $vars An array of license keys (all fields optional) including:
	 *
	 */
	public function update($license_keys_id, array $vars) {
		// Validate license_keys_id
		$vars['license_keys_id'] = $license_keys_id;
        if (isset($vars['service_id']) && $vars['service_id']==='') {
            $vars['service_id']=NULL;
        }
        if (isset($vars['renewal_date']) && $vars['renewal_date']==='') {
            $vars['renewal_date']=NULL;
        }

        //$fields = ['first_name', 'last_name', 'email', 'client_id', 'client_id_code','extension_name','version','installation_url','ip_address','license_code','subscribed'];
		$fields = ['client_id', 'client_id_code','extension_name','version','installation_url','ip_address','license_id','license_code','blesta_license_code','service_id','support_service_id','support_service_status','renewal_date','subscribed'];
		$this->Record->where('license_keys_id', '=', $license_keys_id)->update('license_keys', $vars, $fields);
	}

    /**
     * Update Support Service Status
     * @param $support_service_id
     * @param $support_service_status
     * @return void
     */
    public function updateSupportStatus($support_service_id,$support_service_status) {
         $this->Record->where('support_service_id', '=', $support_service_id)->update('license_keys',['support_service_status'=>$support_service_status]);
    }

    /**
     * Update Rent Service Status
     * @param $service_id
     * @param $service_status
     * @return void
     */
    public function updateRentStatus($service_id,$support_service_status) {
        $this->Record->where('service_id', '=', $service_id)->update('license_keys',['support_service_status'=>$support_service_status]);
    }

    /**
     * @param $license_keys_id
     * @return mixed
     */
	public function getLicenseInfoByLicenseKeyId($license_keys_id) {
        return $this->Record->query( "SELECT * from `license_keys` where `license_keys_id`=?", $license_keys_id)->fetch();
    }

    /**
     * Update rent renewal date
     * @param $extension_name
     * @param $service_id
     * @param $renewal_date
     * @param $renewed_date
     */
	public function updateRenewDate($extension_name,$service_id,$renewal_date,$renewed_date) {
        $this->Record->where('extension_name', 'like', '%'.$extension_name.'%')->where('service_id', '=', $service_id)->update('license_keys', array('renewal_date'=>$renewal_date,'renewed_date'=>$renewed_date));
    }

    /**
     * Update support service renewal date
     * @param $extension_name
     * @param $support_service_id
     * @param $renewal_date
     * @param $renewed_date
     */
    public function updateSupportRenewDate($extension_name,$support_service_id,$renewal_date,$renewed_date) {
        $this->Record->where('extension_name', 'like', '%'.$extension_name.'%')->where('support_service_id', '=', $support_service_id)->update('license_keys', array('renewal_date'=>$renewal_date,'renewed_date'=>$renewed_date));
    }


    /**
     * @param $service_id
     * @param $renewal_date
     * @return void
     */
    public function updateRenewalDate($service_id,$renewal_date) {
        $this->Record->where('service_id', '=', $service_id)->update('license_keys', array('renewal_date'=>$renewal_date));
    }

    /**
     * @param $extension_name
     * @param $client_id
     * @param $service_id
     * @param $renewal_date
     * @return mixed
     */
    public function updateServiceId($extension_name,$client_id,$service_id,$renewal_date) {
        return $this->Record->where('extension_name', 'like', '%'.$extension_name.'%')->where('client_id','=',$client_id)->where('service_id','=',$service_id)->update('license_keys', array('renewal_date'=>$renewal_date));
    }

    /**
     * @param $service_id
     * @return mixed
     */
    public function getLicenseId($service_id) {
        return $this->Record->query( "SELECT license_id,license_code,blesta_license_code,renewed_date from license_keys where `service_id`=? ", (int)$service_id)->fetch();
    }

    /**
     * @return mixed
     */
    public function getLastId() {
        return $this->Record->query( "SELECT MAX(`service_id`) from license_keys ")->fetch();
    }

    /**
     * @param $client_id
     * @param $extension_name
     * @return mixed
     */
    public function getLicensesByClientId($client_id,$extension_name) {
        return $this->Record->query( "SELECT * from license_keys where client_id=? and extension_name=? and license_id <> 0", $client_id,$extension_name)->fetch();
    }

    /**
     * @param $license_code
     * @param $url
     * @return mixed
     */
    public function getLicensesInfoByLicenseCode($license_code) {
        return $this->Record->query( "SELECT * from license_keys where license_code=? ", $license_code)->fetch();
    }

    /**
     * @param $blesta_license_code
     * @param $url
     * @return mixed
     */
    public function getLicensesInfoByBlestaLicenseCode($blesta_license_code) {
        return $this->Record->query( "SELECT * from license_keys where blesta_license_code=? ", $blesta_license_code)->fetch();
    }

    /**
     * @param $service_id
     * @return mixed
     */
    public function getLicenseKeysId($service_id) {
        return $this->Record->select('license_keys_id')->from('license_keys')->where('service_id', '=', $service_id)->fetch();
    }

    /**
     * @param $service_id
     * @return mixed
     */
    public function getLicenseCode($license_code) {
        return $this->Record->query( "SELECT COUNT(license_code) as total from license_keys where license_code=?", $license_code)->fetch();
    }

    /**
     * @param $service_id
     * @return mixed
     */
    public function getBlestaLicenseCode($blesta_license_code) {
        return $this->Record->query( "SELECT COUNT(blesta_license_code) as total from license_keys where blesta_license_code=?", $blesta_license_code)->fetch();
    }

    /**
     * @param $client_id
     * @return mixed
     */
    public function getLicenseInfos($client_id) {
        $this->Record->select()->from("license_keys")->where("client_id","=", $client_id);
        return $this->Record;
    }

    /**
     * @param $client_id
     * @param int $page
     * @param string[] $order
     * @return mixed
     */
    public function getLicenseInfo($client_id,$page = 1, $order = ['extension_name' => 'asc']) {
        $this->Record = $this->getLicenseInfos($client_id);
        $licenses_list = $this->Record->order($order)
            ->limit( $this->getPerPage(), ( max( 1, $page ) - 1 ) * $this->getPerPage() )
            ->fetchAll();

        return $licenses_list;
    }

    /**
     * Returns all infos
     *
     *  (if not given will fetch infos for ALL extensions)
     * @return int The number of results
     */
    public function getLicenseCount($client_id) {
        $this->Record = $this->getLicenseInfos($client_id);
        return $this->Record->numResults();
    }

    /**
     * @param $extension_name
     * @return mixed
     */
    public function getClientIdByExtensionName($extension_name) {
        return $this->Record->query( "SELECT client_id from license_keys where extension_name=?", $extension_name)->fetchAll();
    }

    /**
     * @return mixed
     */
    public function getClientIds() {
        return $this->Record->query( "SELECT DISTINCT lk.client_id,ct.email,lk.extension_name from license_keys lk
                                      LEFT JOIN contacts ct ON ct.client_id=lk.client_id  ")->fetchAll();
    }

    /**
     * @param $installation_url
     * @param $extension_name
     * @return mixed
     */
    public function getInstallation($installation_url,$extension_name) {
        //return $this->Record->query( "SELECT COUNT(license_code) as total from license_keys where extension_name=? and installation_url like %?% ", $extension_name,$installation_url)->fetch();
        //return $this->Record->select()->from("license_keys")->where("extension_name","=",$extension_name)->where("installation_url","=", "$installation_url")->fetch();
        return $this->Record->select()->from("license_keys")->where("extension_name","=",$extension_name)->like("installation_url", "%".$installation_url."%")->fetchAll();
    }

	/**
	 * @param int $license_keys_id The license keys ID to permanently remove from the system
	 */
	public function delete($license_keys_id) {
		$this->Record->from('license_keys')->where('license_keys_id', '=', $license_keys_id)->delete();
	}

    /**
     * Delete License Key By Service ID
     * @param $service_id
     * @return void
     */
    public function deleteByServiceId($service_id) {
        $this->Record->from('license_keys')->where('service_id', '=', $service_id)->delete();
    }

	/**
	 * Validates whether the given email fields are acceptable
	 *
	 * @param array $vars An array of email fields to validate
	 * @return bool True if the data is valid, or false otherwise
	 */
	public function validate(array $vars) {
		$this->Input->setRules($this->getRules(false));
		return $this->Input->validates($vars);
	}


	/**
	 * Retrieves the rules for validating email fields
	 *
	 * @param bool $require_job True to validate the job, otherwise validates only email fields
	 * @return array An array of input validation rules
	 */
	private function getRules($require_job = true) {
		$rules = [
			//'from_name' => [
			//	'empty' => [
			//		'rule' => ['isEmpty'],
			//		'negate' => true,
			//		'message' => $this->_('AdminSendEmails.!error.from_name.empty')
			//	]
			//],
			//'from_address' => [
			//	'valid' => [
			//		'rule' => ['isEmail'],
			//		'message' => $this->_('AdminSendEmails.!error.from_address.valid')
			//	]
			//],
			'subject' => [
				'empty' => [
					'rule' => ['isEmpty'],
					'negate' => true,
					'message' => $this->_('AdminSendEmails.!error.subject.empty')
				]
			],
			'html' => [
				'empty' => [
					'rule' => ['isEmpty'],
					'negate' => true,
					'message' => $this->_('AdminSendEmails.!error.html.empty')
				]
			],
		];


		return $rules;
	}

	/**
	 * Generates a parsed sample subject, HTML, and text content of an email
	 * based on the given contact and email information
	 *
	 * @param array $email An array containing the email content:
	 *  - subject The email subject
	 *  - html The email HTML content
	 *  - text The email text content
	 */
	public function getSample(array $email)	{
		$content = (object)[
			'subject' => $email['subject'],
			'html' => $email['html'],
		];

		return $content;
	}

    /**
     * @param array $vars
     * @return string
     */
    public function createWpUser(array $vars) {
        // Add a client
        $fields = ['client_id', 'wp_user_id','status'];
        $this->Record->insert('license_keys_howto', $vars, $fields);
        $info_id = $this->Record->lastInsertId();
        return $info_id;
    }

    public function getWpUserID($client_id) {
        return $this->Record->query( "SELECT wp_user_id from license_keys_howto where client_id=?", $client_id)->fetch();
    }

}