<?php
/**
 * License Manager parent model
 * 
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseManagerModel extends AppModel {
	
	public function __construct() {
		parent::__construct();
		
		Configure::load("license_manager", dirname(__FILE__) . DS . "config" . DS);
		
		Loader::loadModels($this, array("Services", "ModuleManager"));
	}
	
	
	/**
	 * Fetch service info for the given with the given license key
	 *
	 * @param string $license_key The license key to fetch the service on
	 * @return mixed A stdClass object representing the service, false if no such service exists
	 */
	public function getServiceInfo($license_key) {
		
		// Lookup module ID
		$module = $this->getModuleInfo();
		
		if (!$module)
			return false;
		
		$result = $this->Services->searchServiceFields($module->id, Configure::get("LicenseManager.service_key_field"), $license_key);
		
		if (isset($result[0]))
			return $result[0];
		return false;
	}
	
	/**
	 * Fetch module info for the license module
	 *
	 * @return mixed A stdclass object representing the module, false if no such module exists
	 */
	protected function getModuleInfo() {
		// Lookup module ID
		return $this->Record->select(array("modules.*"))->from("modules")->
			where("modules.company_id", "=", Configure::get("Blesta.company_id"))->
			where("modules.class", "=", Configure::get("LicenseManager.module"))->fetch();
	}
	
	/**
	 * Returns an array of service fields for the given service
	 *
	 * @param int $service_id The ID of the service to fetch fields for
	 * @return array An array containing service fields
	 */
	protected function getServiceFields($service_id) {
		$service = $this->Services->get($service_id);
		
		if ($service)
			return $service->fields;
		return array();
	}
	
	
	/**
	 * Converts numerically indexed service field arrays into an object with member variables
	 *
	 * @param array $fields A numerically indexed array of stdClass objects containing keys and values
	 * @return stdClass A stdClass objects with member variables
	 */
	protected function serviceFieldsToObject(array $fields) {
		$data = new stdClass();
		foreach ($fields as $field)
			$data->{$field->key} = $field->value;
		
		return $data;
	}
}
?>