<?php
/**
 * License Manager trial request controller
 * 
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class Trial extends LicenseManagerController {
	
	public function preAction() {
		// Disable CSRF for requests
		Configure::set("Blesta.verify_csrf_token", false);
		
		parent::preAction();
	}
	
	/**
	 * Attempt to issue a trial license
	 */
	public function index() {
		$this->uses(array("Services"));
		
		$license_key = null;
		
		if (!isset($this->post['domain'])) {
			echo $license_key;
			return false;
		}
		
		$exists = $this->isLogged();
		
		if ($exists !== false) {
			echo $exists;
			return false;
		}
	
		// Add the trial
		$vars = array(
			'package_group_id' => Configure::get("LicenseManager.trial_package_group_id"),
			'pricing_id' => Configure::get("LicenseManager.trial_package_pricing_id"),
			'client_id' => Configure::get("LicenseManager.trial_client_id"),
			'status' => 'active',
			'date_canceled' => date("c", strtotime("+" . Configure::get("LicenseManager.trial_length")))
		);
		$service_id = $this->Services->add($vars, array());
		
		if ($service_id) {
			$service = $this->Services->get($service_id);
			
			foreach ($service->fields as $field) {
				if ($field->key == Configure::get("LicenseManager.service_key_field")) {
					$license_key = $field->value;
					break;
				}
			}
		}
		
		if ($license_key) {
			// Write request to log
			$data = array(date("c"), $_SERVER['REMOTE_ADDR'], isset($this->post['domain']) ? $this->post['domain'] : null, $license_key);
			$this->log($data);
		}
		
		echo $license_key;
		return false;
	}
	
	/**
	 * Log trial license
	 *
	 * @param array $data An array of log data
	 */
	private function log($data) {
		if (!isset($this->LicenseManagerSettings))
			$this->uses(array("LicenseManager.LicenseManagerSettings"));
		$log_path = $this->LicenseManagerSettings->getSetting("log_trial_file", $this->company_id);
		
		if (!empty($log_path['log_trial_file']))
			file_put_contents($log_path['log_trial_file'], implode("\t", $data) . "\n", FILE_APPEND);
	}
	
	/**
	 * Determines if the given domain and/or IP address is logged.
	 * IP must not be logged within the last hour, domain must not ever be logged
	 *
	 * @return mixed null if the IP address is logged within the last hour, string containing the license key if the domain exists, boolean false if no records exist.
	 */
	private function isLogged() {
		$recent = false;
		
		if (!isset($this->LicenseManagerSettings))
			$this->uses(array("LicenseManager.LicenseManagerSettings"));
		$log_path = $this->LicenseManagerSettings->getSetting("log_trial_file", $this->company_id);
		
		if (empty($log_path['log_trial_file']))
			return false;
		
		$delimiter = "\t";
		
		// Evaluate logs to see if trial is possible
		$fh = fopen($log_path['log_trial_file'], "r");
		while (($data = fgetcsv($fh, 0, $delimiter)) !== false) {
			list($date, $ip, $domain, $license_key) = $data;

			if (strtolower($domain) == strtolower($this->post['domain']) && strtotime($date) >= strtotime(date("c") . "-90 days"))
				return $license_key;			
			if ($ip == $_SERVER['REMOTE_ADDR'] && strtotime($date) >= strtotime(date("c") . "-1 hour"))
				$recent = true;
		}
		fclose($fh);
		
		if ($recent)
			return null;
		
		return false;
	}
}