<?php
/**
 * An example integration of the Blesta License Manager plugin in an
 * application. Feel free to use this within your application.
 *
 * @license http://opensource.org/licenses/mit-license.php MIT License
 * @copyright Copyright (c) 2014, Phillips Data Inc.
 */
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';
require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . "license.php";

$server_url = "http://localhost/blesta/plugin/license_manager/validate/";
$license_key = "no2zg4og5id7an47"; // The client's license key
$public_key = ""; // The client's public key (if one exists yet)
$shared_secret = "The quick brown fox jumps over the lazy dog"; // A random shared secret value that exists for this Licese Module product
$path_to_phpseclib = dirname(__FILE__) . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "phpseclib" . DIRECTORY_SEPARATOR; // The path to the phpseclib library
$product_version = "1.0.0"; // The version of your software product

// Initialize the License Manager
$license = new License($path_to_phpseclib);
$license_manager = $license->getManager();
$license_manager->setLicenseServerUrl($server_url);
$license_manager->setKeys($license_key, $public_key, $shared_secret);

// Fetch the public key
$public_key = $license_manager->requestKey();
var_dump($public_key);

# NOTE: Store the public key somewhere so you don't have to refetch it

// Now that we have a public key, we can send and receive communications
$license_manager->setKeys($license_key, $public_key, $shared_secret);
$custom_data = array('version' => $product_version); // An array of anything you want to send to the license server. 'version' is the only custom field currently saved for the service
$license_data = $license_manager->requestData($custom_data);
var_dump($license_data);

# NOTE: Store the license data somewhere and refetch it when necessary (e.g. when an admin user logs in, or a cron runs)
# NOTE: If, for some reason, we received no license data refetch the public key (it may have changed)

// Now that we have our license data, we can evaluate it
$ttl = 1209600; // Amount of time local license data should be valid for (60*60*24*14 = 1209600 = 14 days)
$data = $license_manager->validate($license_data, $ttl);
var_dump($data);

# NOTE: Perform the correct action based on the $data['status'] value (i.e. if expired, refetch license data, etc.)

#
# IMPORTANT: Be sure to unset any sensitive data immediately after usage!
#
unset($license, $license_manager, $public_key, $shared_secret, $server_url, $license_key);
?>