<?php
/**
 * License Manager Settings
 * 
 * @package blesta
 * @subpackage blesta.plugins.license_manager
 * @copyright Copyright (c) 2010, Phillips Data, Inc.
 * @license http://www.blesta.com/license/ The Blesta License Agreement
 * @link http://www.blesta.com/ Blesta
 */
class LicenseManagerSettings extends LicenseManagerModel {
	
	public function __construct() {
		parent::__construct();
		
		if (!isset($this->SettingsCollection))
			Loader::loadComponents($this, array("SettingsCollection"));
	}
	
	/**
	 * Fetch a particular setting
	 *
	 * @param string $key The setting key
	 * @param int $company_id The ID of the company to fetch the setting for
	 * @return array containing the key and value for this setting
	 */
	public function getSetting($key, $company_id) {
		$setting = $this->SettingsCollection->fetchSettings(null, $company_id, "license_manager." . $key);
		if ($setting) {
			$setting[$key] = $setting["license_manager." . $key];
			unset($setting["license_manager." . $key]);
		}
		return $setting;
	}

	/**
	 * Fetch a list of settings
	 *
	 * @param int $company_id The ID of the company to fetch settings for
	 * @return array A key=>value array of system settings
	 */	
	public function getSettings($company_id) {
		$settings = array();
		$all_settings = $this->SettingsCollection->fetchSettings(null, $company_id);
		foreach ($all_settings as $setting => $value) {
			if (substr($setting, 0, strlen("license_manager.")) == "license_manager.")
				$settings[substr($setting, strlen("license_manager."))] = $value;
		}
		return $settings;		
	}
	
	/**
	 * Set settings
	 *
	 * @param int $company_id The company ID
	 * @param array $settings Settings to set as key/value pairs
	 */
	public function setSettings($company_id, array $settings) {
		if (!isset($this->Companies))
			Loader::loadModels($this, array("Companies"));
		
		$valid_settings = array(
			"log_file", "log_trial_file", "log_format"
		);
		
		$set = array();
		foreach ($settings as $key => $value) {
			if (in_array($key, $valid_settings))
				$set["license_manager." . $key] = $value;
		}
		$this->Companies->setSettings($company_id, $set);
	}
}
?>